@@ -14,10 +14,9 @@
 use crate::transport::Error;
 #[cfg(feature = "zmq")]
 use crate::zmqsocket;
-use crate::{
-    session, LocalNode, LocalSocketAddr, NodeAddr, RemoteNodeAddr,
-    RemoteSocketAddr, Session,
-};
+use crate::{session, LocalNode, LocalSocketAddr, NodeAddr, Session};
+#[cfg(feature = "keygen")]
+use crate::{RemoteNodeAddr, RemoteSocketAddr};
 
 pub trait Connect {
     fn connect(&self, node: &LocalNode) -> Result<Box<dyn Session>, Error>;
@@ -65,6 +64,7 @@ impl Accept for LocalSocketAddr {
     }
 }
 
+#[cfg(feature = "keygen")]
 impl Connect for RemoteNodeAddr {
     fn connect(&self, local: &LocalNode) -> Result<Box<dyn Session>, Error> {
         Ok(match self.remote_addr {
@@ -94,6 +94,7 @@ impl Connect for RemoteNodeAddr {
     }
 }
 
+#[cfg(feature = "keygen")]
 impl Accept for RemoteNodeAddr {
     fn accept(&self, local: &LocalNode) -> Result<Box<dyn Session>, Error> {
         Ok(match self.remote_addr {
@@ -123,7 +124,12 @@ impl Connect for NodeAddr {
     fn connect(&self, local: &LocalNode) -> Result<Box<dyn Session>, Error> {
         match self {
             NodeAddr::Local(addr) => addr.connect(local),
+            #[cfg(feature = "keygen")]
             NodeAddr::Remote(addr) => addr.connect(local),
+            #[cfg(not(feature = "keygen"))]
+            NodeAddr::Remote(addr) => {
+                Err(Error::KeygenFeatureRequired("encrypted TCP connect"))
+            }
         }
     }
 }
@@ -132,7 +138,12 @@ impl Accept for NodeAddr {
     fn accept(&self, local: &LocalNode) -> Result<Box<dyn Session>, Error> {
         match self {
             NodeAddr::Local(addr) => addr.accept(local),
+            #[cfg(feature = "keygen")]
             NodeAddr::Remote(addr) => addr.accept(local),
+            #[cfg(not(feature = "keygen"))]
+            NodeAddr::Remote(addr) => {
+                Err(Error::KeygenFeatureRequired("encrypted TCP accept"))
+            }
         }
     }
 }
