@@ -11,11 +11,14 @@
 // along with this software.
 // If not, see <https://opensource.org/licenses/MIT>.
 
-mod act;
-mod chacha;
-mod conduit;
+mod ceremony;
+pub mod chacha;
 mod handshake;
 mod hkdf;
+mod transcoder;
 
-pub use conduit::{NoiseDecryptor, NoiseEncryptor, NoiseTranscoder};
 pub use handshake::{HandshakeError, HandshakeState};
+pub use transcoder::{
+    NoiseDecryptor, NoiseEncryptor, NoiseTranscoder,
+    MESSAGE_LENGTH_HEADER_SIZE, TAGGED_MESSAGE_LENGTH_HEADER_SIZE,
+};
